'use strict';

const HTTPStatus = require('http-status');
const logger = require('../../../logger');
const _ = require('lodash');
const config = require('../../../configuration/index');
const SchemaValidationError = require('../../../utils/SchemaValidationError');

class CommandController {

  constructor(dataProvider) {
    this._dataProvider = dataProvider;
  }

  handleCommand(req, res) {
    logger.info('Command controller got request on router: POST /commands/:command');

    let commandInLowerCase = req.params.command.toLowerCase();
    let supportedCommands = Object.keys(config.commandServer.commands);

    if(!supportedCommands.includes(commandInLowerCase)) {
      throw new SchemaValidationError('command parameter has invalid value');
    }

    let options = {
      panelId: req.body.panelId,
      doorIds: req.body.doors,
      command: commandInLowerCase
    };
    this._dataProvider.sendCommand(options);
    res.status(HTTPStatus.NO_CONTENT).send();
  };

  handleCommands(req, res) {
    logger.info('Command controller got request on router: POST /commands');

    for(let command of req.body.commands) {
      let options = {
        panelId: req.body.panelId,
        doorIds: req.body.doors,
        command: command.toLowerCase()
      };
      this._dataProvider.sendCommand(options);
    }
    res.status(HTTPStatus.NO_CONTENT).send();
  };
}

module.exports = CommandController;