'use strict';

const express = require('express');
const app = express();
const bodyParser = require('body-parser');
const utils = require('../utils/server-utils');
const CommandController = require('./api/commands/command-controller');
const logger = require('../logger');
const requestValidator = require('../utils/request-validator');
const requestSchemas = require('./api/commands/command.jsonschema.json');

function startServer(dataProvider, port) {
  let commandController = new CommandController(dataProvider);

  app.use(bodyParser.urlencoded({ extended: false }));
  app.use(bodyParser.json());

  /**
   * POST / apply list of commands on the list of doors
   */
  app.post('/commands',
      requestValidator.validate(requestSchemas.postCommands),
      commandController.handleCommands.bind(commandController));

  /**
   * POST / apply specified command on the list of doors
   */
  app.post('/commands/:command',
      requestValidator.validate(requestSchemas.postCommand),
      commandController.handleCommand.bind(commandController));

  app.use(utils.errorHandler);

  let server = app.listen(port);
  logger.info(`Start command server on port: ${port}`);

  // It should return server instance for testing purposes
  return server;
}

module.exports = {
  startServer
};