'use strict';

/**
 * The purpose of this function is to return user authenticator decorator that will encapsulate logic for refresh token
 * update on file system
 * @param authenticator User authentication flow object from PDK client
 * @param fileAdapter file adapter instance to store refresh token in the file system after renew
 */
module.exports.wrapUserAuthenticator = (authenticator, fileAdapter) =>
  async () => {
    // original token_set function from pdk client
    let tokenSetFunc = await authenticator();

    let tokenSetWrapper = async () => {
      return await tokenSetFunc();
    };

    tokenSetWrapper.refresh = async () => {
      await tokenSetFunc.refresh();
      // Get new token set and store refresh token on the file system
      let newSet = await tokenSetFunc();

      // save new refresh token in the filesystem
      await fileAdapter.setToken(newSet.refresh_token);
    };

    tokenSetWrapper.revoke = tokenSetFunc.revoke;

    // Store initially refreshed token
    let initialTokenSet = await tokenSetFunc();
    await fileAdapter.setToken(initialTokenSet.refresh_token);

    return tokenSetWrapper;
  };
