'use strict';

const randomString = require('random-string');
const _ = require('lodash');
const config = require('../configuration/index');
const HTTPStatus = require('http-status');
const logger = require('../logger');

/**
 * Transform arrived command to necessary format
 */
module.exports.populateCommand = (doorId, command) => {
  return {
    topic: config.commandServer.commands[command],
      body: {
        doorId: doorId
      },
    id: randomString({length: 32})
  }
};

/**
 * Common error handler for express chain.
 */
module.exports.errorHandler = function (err, req, res, next) {

  logger.error('errorHandler', err.message);
  let errorDetails = {};

  if (err.name === 'JsonSchemaValidation') {
    err.status = HTTPStatus.BAD_REQUEST;
    errorDetails.message = err.message + ' ' + JSON.stringify(err.validations);
  }

  errorDetails.message = errorDetails.message || 'unexpected server error';

  res.status(err.status || HTTPStatus.INTERNAL_SERVER_ERROR);
  res.json(errorDetails);
};